// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>The <tt>Runtime</tt> class provides the bootstrapping feature for a CORBA object implemented
 **  using the system component based framework.</p>
 **/
public class Runtime
{
    static private String _class_name = "Runtime";
    static private FactoryFinderService _ff_service;
    static private RegistrationService _reg_service;
    static private MainThread _main_thread;
    // NOTE: stores the main orb instance
    static private org.omg.CORBA.ORB _orb;

    static {
        // create FactoryFinder service
        _ff_service = new FactoryFinderServiceImpl();
        // complete configuration
        _ff_service.system_configuration_complete(new SystemConfigurationImpl());

        // create Registration service
        _reg_service = new RegistrationServiceImpl();
        // complete configuration
        _reg_service.system_configuration_complete(new SystemConfigurationImpl());

        // register ORB service factory
        String entrypt = "org.objectweb.corba.runtime.ORBFactoryImpl.create_factory";
        _ff_service.register_service_factory(ORBService.SERVICE_ID, null, entrypt);

        // add debug PI to the ORB service factory
        ORBFactory fact = (ORBFactory)getFFService().find_service_factory(ORBService.SERVICE_ID);
        fact.add_default_initializer("org.objectweb.corba.runtime.DebugORBInitializerImpl");

        // register POA service factory
        entrypt = "org.objectweb.corba.runtime.POAFactoryImpl.create_factory";
        _ff_service.register_service_factory(POAService.SERVICE_ID, null, entrypt);

        // create new main thread
        _main_thread = new SingleORBWorker();
        _main_thread.startThread();

        _orb = null;
    }

    //
    //  public operations
    //

    static public FactoryFinderService
    getFFService()
    {
        return _ff_service;
    }

    static public RegistrationService
    getRegistrationService()
    {
        return _reg_service;
    }

    static public MainThread
    getMainThread()
    {
        return _main_thread;
    }

    static public org.omg.CORBA.ORB
    getORB()
    {
        return _orb;
    }

    static public Servant
    loadServant(RuntimeConfiguration config)
    {
        final String opname = "loadServant";

        // register default factories for services
        String[] sids = config.service_ids;
        String[] entrypts = config.service_entrypts;

        for (int i=0;i<sids.length;i++) {
            _ff_service.register_service_factory(sids[i], null, entrypts[i]);
        }

        // add archive (JAR) location to the class loader path
        String archloc = config.archive_location;

        // check location
        if (archloc!=null) {
            TheClassLoader.addResource(archloc);
        }

        // create new instance
        String entrypt = config.archive_entrypoint;
        Object obj = TheClassLoader.newInstance(entrypt);

        // check creation
        if (obj==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (servant not created)";
            TheLogger.error(_class_name, opname, msg);
        }

        // check that the instance is a Servant object
        Servant servant = null;
        try {
            servant = (Servant)obj;
        } catch (ClassCastException ex) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (not a servant)";
            TheLogger.error(_class_name, opname, msg);
        }

        // create system services
        // create ORB service instance
        ORBService orbs = null;
        SystemFactory orbfact = _ff_service.find_service_factory(ORBService.SERVICE_ID);

        // create config
        ORBConfiguration orbcfg = new ORBConfigurationImpl(config.orb_initializers,
                                                           config.value_factories,
                                                           config.initial_references,
                                                           config.stringified_initial_references);

        orbs = (ORBService)orbfact.create_system_component(orbcfg);

        // set as ORB singleton
        try {
            ORBServiceImpl orbimpl = (ORBServiceImpl)orbs;
            _orb = orbimpl.getORB();
        } catch (ClassCastException ex) {
            // ignore
        }

        // create POA service instance
        // NOTE: TODO: this POA is used to create the servant ref and shoud therefore be 
        // configurable externally like the ORB
        POAService poas = null;
        SystemFactory poafact = _ff_service.find_service_factory(POAService.SERVICE_ID);

        // create config 
        // params: no name, no pols, orbs
        POAConfiguration poacfg = new POAConfigurationImpl(null, null, orbs);
        poas = (POAService)poafact.create_system_component(poacfg);

        // complete configuration
        ServantConfiguration scfg = new ServantConfigurationImpl(orbs, poas);
        servant.system_configuration_complete(scfg);

        // create reference and register it
        // use runtime.id as oid
        byte[] oid = System.getProperty("runtime.id").getBytes();
        org.omg.CORBA.Object ref = poas.create_ref_with_id(oid, servant.type_id());

        // activate servant
        poas.activate_servant_with_id(servant, oid);

        // try to store in local file and in ns
        RegistrationService regs = _reg_service;
        // check if a IOR file is set
        String iorfile = System.getProperty("runtime.iorfile");
        if (iorfile!=null) {
            // obtain file registration scheme
            FileRegistrationScheme filescheme = (FileRegistrationScheme)regs.get_scheme(FileRegistrationScheme.SCHEME_ID);
            filescheme.write_ior(iorfile, ref, orbs);
        }

        // obtain ns name from properties (always set)
        String insname = System.getProperty("runtime.id");
        INSRegistrationScheme insscheme = (INSRegistrationScheme)regs.get_scheme(INSRegistrationScheme.SCHEME_ID);
        insscheme.bind(insname, ref, orbs);

        return servant;
    }

    static public void
    loadMain(RuntimeConfiguration config)
    {
        // add archive (JAR) location to the class loader path
        String archloc = config.archive_location;

        // check location
        if (archloc!=null) {
            TheClassLoader.addResource(archloc);
        }

        // create ORB service instance
        ORBService orbs = null;
        SystemFactory orbfact = _ff_service.find_service_factory(ORBService.SERVICE_ID);

        // create config
        ORBConfiguration orbcfg = new ORBConfigurationImpl(config.orb_initializers,
                                                           config.value_factories,
                                                           config.initial_references,
                                                           config.stringified_initial_references);

        orbs = (ORBService)orbfact.create_system_component(orbcfg);

        // NOTE: the expected signature is :
        //    static public java.lang.Runnable
        //    <op_name>(ORBService orbs);
        //
        String entrypt = config.archive_entrypoint;
        Class[] argc = { ORBService.class };
        Object[] argv = { orbs };

        Runnable run = null;
        try {
            run = (Runnable)TheClassLoader.newInstance(entrypt, argc,  argv);
        } catch (ClassCastException ex) {
            // NOTE: TODO: should be reported as an exception
            final String opname = "loadMain";
            final String msg = "FAILED (not a runnable)";
            TheLogger.error(_class_name, opname, msg);
        }

        run.run();
    }
}
