// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>The <tt>RuntimeMain</tt> class allows to boot a CORBA object (implemented using the
 **  system component framework) using the <tt>Runtime.loadServant</tt> operation. The parameters
 **  are extracted from the command line information.</p>
 **
 **  <p>In its current state, the following information must/may be provided:</p>
 **  <li>
 **     <ul>Servant JAR location (mandatory) through the Java <tt>archive.location</tt> system property or
 **         through the "-ArchiveLocation location" option on the command line.</ul>
 **     <ul>Servant entrypoint (mandatory) through the Java <tt>archive.entrypoint</tt> system property or
 **         through the "-ArchiveEntrypoint entrypt" option on the command line.</ul>
 **     <ul>Initial references (optional) through "-ORBInitRef regname=ior" option on the command line.</ul>
 **     <ul>System services (optional) through "-SystemService serviceid=entrypoint" option on the command line.</ul>
 **     <ul>Runtime identity (mandatory) through the Java <tt>runtime.id</tt> system property or
 **         through the "-RuntimeId rid" option on the command line.</ul>
 **  </li>
 **
 **  @see org.objectweb.corba.runtime.Runtime Runtime
 **/
public class RuntimeMain
{
    static final private String _class_name = "RuntimeMain";

    //
    //  public operation
    //

    static public RuntimeConfiguration
    parseConfiguration(String[] args)
    {
        String opname = "parseConfiguration";

        RuntimeConfiguration config = new RuntimeConfiguration();

        // obtain initial references passed on the command line
        // and default system services
        java.util.Vector virefs = new java.util.Vector();
        java.util.Vector vservices = new java.util.Vector();

        // obtain archive location and entrypoint
        // NOTE: may also be set with java properties
        // get root object archive location and entrypoint from System properties
        String location = System.getProperty("archive.location");
        String entrypt = System.getProperty("archive.entrypoint");

        // parse args
        for (int i=0;i<args.length;i++) {
            if (args[i].equals("-ORBInitRef")) {
                virefs.add(args[i+1]);
                i++;
            }

            if (args[i].equals("-SystemService")) {
                vservices.add(args[i+1]);
                i++;
            }

            if (args[i].equals("-ArchiveLocation")) {
                location = args[i+1];
                i++;
            }

            if (args[i].equals("-ArchiveEntrypoint")) {
                entrypt = args[i+1];
                i++;
            }

            if (args[i].equals("-RuntimeId")) {
                // set as a property
                System.setProperty("runtime.id", args[i+1]);
                i++;
            }
        }
    
        // parse initial references
        String[] irefs = (String[])virefs.toArray(new String[0]);
        StringifiedInitialReference[] sirefs = new StringifiedInitialReference[irefs.length];

        int idx = 0;
        for (int i=0;i<irefs.length;i++) {
            // normally, an initial reference has the following format:
            //   "<reg_name>=<ref_ior>"

            //String msg = "initial reference found: "+irefs[i];
            //TheLogger.debug(_class_name, opname, msg);

            // look for the "="
            idx = irefs[i].indexOf("=");
            sirefs[i] = new StringifiedInitialReference();
            sirefs[i].name = irefs[i].substring(0, idx);
            sirefs[i].ref = irefs[i].substring(idx+1);
        }

        // store
        config.stringified_initial_references = sirefs;

        // parse default system services
        String[] services = (String[])vservices.toArray(new String[0]);
        String[] service_ids = new String[services.length];
        String[] service_entrypts = new String[services.length];

        idx = 0;
        for (int i=0;i<services.length;i++) {
            // normally, an default system service has the following format:
            //   "<service_id>=<entrypoint>"

            //String msg = "default system service found: "+services[i];
            //TheLogger.debug(_class_name, opname, msg);

            // look for the "="
            idx = services[i].indexOf("=");
            service_ids[i] = services[i].substring(0, idx);
            service_entrypts[i] = services[i].substring(idx+1);
        }

        // store
        config.service_ids = service_ids;
        config.service_entrypts = service_entrypts;

        // NOTE: if the location is non-local, the archive should be downloaded first
        // not necessary at this stage, but should be done
        config.archive_location = location;
        config.archive_entrypoint = entrypt;

        //
        return config;
    }

    //
    //  main
    //

    static public void
    main(String[] args)
    {
        // log a start message
        String opname = "main";
        TheLogger.debug(_class_name, opname, "Loading ...");

        // parse config
        RuntimeConfiguration config = parseConfiguration(args);

        // check if entrypoint ends with ".main" 
        // in that case, it's not a servant, but a main 
        // NOTE: it's just a dirty code to be able to start main without requiring
        // servant types and so on
        if (config.archive_entrypoint.endsWith(".main")) {
            Runtime.loadMain(config);
            // log a end message
            TheLogger.debug(_class_name, opname, "Main loaded");
        }
        else {
            // load servant
            Servant serv = Runtime.loadServant(config);
            // log a end message
            TheLogger.debug(_class_name, opname, "Servant loaded");
        }

        System.err.println("Launcher waiting for this usefull message ...");
        System.out.println("... or for this one ? :o(");
    }
}
