// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;
// import of OpenCCM modules
import org.objectweb.corba.ast.api.*;
import org.objectweb.corba.ast.lib.*;
import org.objectweb.corba.generator.java.lib.*;
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;
import org.objectweb.corba.generator.common.lib.GenerationException;

/**
 **  <p>Generator class for cidl_idl tool.</p>
 **
 **  <p>NOTE: for each declaration in the root scope of the file, a file containing its IDL mapping is
 **  generated. The filename is built using the IDL absolute name of the declaration and replacing
 **  the "::" string by the "_" character (and removing the first "::") and appending "_seg.idl" string.</p>
 **/
public class CIDLIDLGenerator
extends org.objectweb.corba.generator.common.lib.GeneratorBase
{
    // generator
    private IDLTranslator _translator;
    private String _last_prefix;
    private java.util.Stack _module_stack;

    // default constructor
    public
    CIDLIDLGenerator(AST ast)
    {
        super(ast);

        _translator = new IDLTranslator();
        _last_prefix = null;
        _module_stack = null;
    }

    //
    //  internal operations
    //

    private IDLTranslator
    getTranslator()
    {
        return _translator;
    }

    private java.util.Stack
    computeContentStack(Declaration decl)
    {
        // NOTE: compute a stack of scope starting at the root scope (ie the repository)
        // and ending at the target declaration
        // This allows generating only the required modules (without their full content)
        // until reaching the target declaration
        // NOTE2: the stack does not contain the target decl

        // create tree as an array list
        // NOTE: only one child per parent 
        java.util.Stack stack = new java.util.Stack();

        Declaration curr = decl;
        Declaration parent = decl.getParent();
        while ((parent.getDeclKind()!=DeclarationKind.dk_repository) &&
               !(parent instanceof FileScope)) {
            stack.push(parent);
            curr = parent;
            parent = curr.getParent();
        }

        return stack;
    }

    private void
    generateForDeclaration(Declaration decl)
    throws GenerationException
    {
        // generation is done only for: 
        //  - module
        //  - component
        if (decl.getDeclKind()==DeclarationKind.dk_module) {
            generateForModule((ModuleDecl)decl);
        } else if (decl.getDeclKind()==DeclarationKind.dk_component) {
            generateForComponent((ComponentDecl)decl);
        } 
    }

    private void
    generateForModule(ModuleDecl module)
    throws GenerationException
    {
        put("obj", module);
        map("MODULE");
    }

    private void
    generateForComponent(ComponentDecl comp)
    throws GenerationException
    {
        // NOTE: no CIDL, assume monolithic
        // for a component C1:
        //   component C1 {
        //     provides I1 f_i1;
        //     uses I2 r_i2;
        //     consumes E1 c_e1;
        //     emits E2 e_e2;
        //   };
        // the following interface is generated:
        //   interface C1_MainSeg : C1, I1, E1Consumer {};
        //
        // for a component C2:
        //   component C2 : C1 {};
        // the following interface is generated:
        //   interface C2_MainSeg : C2, I1, E1Consumer {};
        //

        // build the list of inherited interfaces
        StringBuffer list = new StringBuffer();
        Declaration[] decls = null;

        // facets
        decls = comp.getContents(false, DeclarationKind.dk_provides);
        for (int i=0;i<decls.length;i++) {
            ProvidesDecl facet = (ProvidesDecl)decls[i];
            list.append(", "+facet.getInterface().getAbsoluteName());
        }

        // sinks
        decls = comp.getContents(false, DeclarationKind.dk_consumes);
        for (int i=0;i<decls.length;i++) {
            ConsumesDecl sink = (ConsumesDecl)decls[i];
            list.append(", "+sink.getEvent().getAbsoluteName()+"Consumer");
        }

        put("obj", comp);
        put("segment_itf_list", list);
        map("COMPONENT");
    }

    //
    //  public operations for .vm script (must return a string)
    //

    final public String
    getMacro(Declaration decl)
    {
        String res = "";
        String version = decl.getVersion();
        String id = decl.getId();
        String prefix = decl.getPrefix();

        // System.err.println("Macro for " + decl.getAbsoluteName() + " : " + prefix);
        if (!prefix.equals(_last_prefix))
        {
            res = res + "#pragma prefix \""+prefix+"\"\n";
            _last_prefix = prefix;
        }
        else
        {
            String p_id = decl.getParent().getId();
            int idx1 = p_id.indexOf(':');
            int idx2 = p_id.lastIndexOf(':');
            if (id.indexOf(p_id.substring(idx1, idx2))==-1)
                res = res + "#pragma id "+decl.getName()+" \""+id+"\"\n";
        }
        if (!version.equals("1.0"))
            res = res + "#pragma version "+decl.getName()+" "+version+"\n";

        return res;
    }

    final public String
    generateContents(Scope scope)
    throws GenerationException
    {
        Declaration[] decls = scope.getContents(false, DeclarationKind.dk_all);
        for (int i=0;i<decls.length;i++) {
            generateForDeclaration(decls[i]);
        }

        return "";
    }

    final public String
    generateWithContents(java.util.Stack stack)
    throws GenerationException
    {
        // get the top declaration
        Object obj = stack.pop();
        put("obj", obj);

        if (stack.empty()) {
            map("MODULE");
        } else {
            put("content", stack);
            map("MODULE_WITH_CONTENT");
        }

        return "";
    }

    final public String
    pushModule(ModuleDecl module)
    {
        _module_stack.push(module);
        return "";
    }

    final public String
    popModule()
    {
        Object obj = _module_stack.pop();
        put("obj", obj);
        return "";
    }

    //
    //  public operations
    //

    final public void
    initialize(org.objectweb.util.cmdline.api.Console console)
    {
        // ressource path
        java.util.Vector rpath = new java.util.Vector();
        rpath.add(System.getProperty("ecm.templates"));
        setRessourcePath(rpath);

        // templates
        java.util.Vector libs = new java.util.Vector();
        libs.add("cidl_idl.vm");
        setLibrary(libs);

        // call the common generator init method
        //log_enabled_ = true; // set to false by the constructor of GeneratorBase class
        super.init();

        // initialize velocity context
        put("which", "cidl_idl");
        put("generator", this);
    }

    final public void
    generate(String basedir, Declaration decl,
             org.objectweb.util.cmdline.api.Console console)
    throws GenerationException
    {
        console.message("Will generate for: "+decl.getAbsoluteName());


        // build output filename from declaration name by replacing "::" by "_" and appending "_seg.idl"
        // remove the first "::"
        String absname = decl.getAbsoluteName().substring(2);
        String basename = absname.replaceAll("::", "_");
        String filename = basedir+java.io.File.separator+basename+"_seg.idl";
        open(filename, "out");

        // general elements
        _last_prefix = "";
        _module_stack = new java.util.Stack();
        put("FILE_IFDEF_MACRO", "__"+basename.toUpperCase()+"_SEG__");

        // include file with name without "_seg" (as generated by OpenCCM ir3_idl2 generator)
        java.util.ArrayList includes = new java.util.ArrayList();
        includes.add(basename+".idl");
        put("includes", includes);

        map("FILE_HEADER");
        generateWithContents(computeContentStack(decl));
        map("FILE_FOOTER");

        close("out");
        console.message("File '"+filename+"' has been generated.");
    }
}


