// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ecm.taskdefs;

/**
 **  <p>Build a jar archive from a source directory. This task may work in three ways:</p>
 **  <ul>
 **    <li>Compilation of source files only (compile-only mode).</li>
 **    <li>Building of an archive from class files only (build-only mode).</li>
 **    <li>Compilation of source files and building of an archive (twin mode).</li>
 **  </ul>
 **
 **  <p>In the first case, you have to provide the <tt>srcdir</tt>, <tt>compileto</tt> and
 **  <tt>bootclasspath</tt> attributes at least.
 **  In the second case, you have to provide the <tt>archivename</tt> and <tt>buildto</tt>
 **  attributes at least.
 **  In the last case, the five attributes must be provided at least.</p>
 **
 **  <p><bold>Parameters</bold></p>
 **  <tr>
 **    <td><bold>Attribute</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>srcdir</td>
 **    <td>Directory of the source files.</td>
 **    <td>Yes in compile-only and twin modes.</td>
 **  </tr>
 **  <tr>
 **    <td>compileto</td>
 **    <td>Directory of the class files.</td>
 **    <td>Yes in compile-only and twin modes.</td>
 **  </tr>
 **  <tr>
 **    <td>archivename</td>
 **    <td>Name of the built archive.</td>
 **    <td>Yes in build-only and twin modes.</td>
 **  </tr>
 **  <tr>
 **    <td>buildto</td>
 **    <td>Directory of the archive (jar) file.</td>
 **    <td>Yes in build-only and twin modes.</td>
 **  </tr>
 **  <tr>
 **    <td>classpathref (or classpathRef)</td>
 **    <td>The reference to the classpath used for compilation.</td>
 **    <td>No</td>
 **  </tr>
 **  <tr>
 **    <td>bootclasspathref (or bootclasspathRef)</td>
 **    <td>The reference to the bootclasspath used for compilation.</td>
 **    <td>Yes in compile-only and twin modes and unless a nested <tt>bootclasspath</tt>
 **        element is present.</td>
 **  </tr>
 **  <tr>
 **    <td>verbose</td>
 **    <td>Asks for verbose output. Defaults to no.</td>
 **    <td>No</td>
 **  </tr>
 **
 **  <p><bold>Parameters specified as nested elements</bold></p>
 **  <p>This task support the <tt>classpath</tt> and <tt>bootclasspath</tt> attributes also
 **  as nested elements in the form of path-like structures (unlike the attributes which are
 **  path references).</p>
 **
 **  <p>This task support the nested <tt>libraries</tt> fileset-like structures. The included
 **  files will be added to the build archive if any.</p>
 **
 **  <p>This task support the nested <tt>library</tt> (without an 's') element in the form of
 **  a filename. The corresponding file will be added to the build archive if any.</p>
 **
 **  <p>This task support the nested <tt>classes</tt> element in the form of a fileset. Each included
 **  file in this fileset will be added to the archive.</pp>
 **/
public class BuildJarTask
extends org.apache.tools.ant.Task
{
    // for compilation
    private java.io.File _srcdir;
    private java.io.File _classdir;
    private java.io.File _compileto;
    private String _packagedir;
    private org.apache.tools.ant.types.Reference _classpath;
    private org.apache.tools.ant.types.Reference _bootclasspath;
    private boolean _bcpcreated;
    private org.apache.tools.ant.taskdefs.Javac _javac;

    // for build
    private String _archive;    
    private java.io.File _buildto;
    private org.apache.tools.ant.taskdefs.Jar _jar;
    private java.util.ArrayList _library;

    // common
    static private int _UNKNOWN_MODE = 0;
    static private int _COMPILE_ONLY_MODE = 1<<1;
    static private int _BUILD_ONLY_MODE = 1<<2;
    static private int _TWIN_MODE = _COMPILE_ONLY_MODE | _BUILD_ONLY_MODE;
    private int _mode;
    private boolean _verbose;


    // default constructor
    public
    BuildJarTask()
    {
        // for compilation
        _srcdir = null;
        _classdir = null;
        _compileto = null;
        _packagedir = null;
        _classpath = null;
        _bootclasspath = null;
        _bcpcreated = false;
        _javac = null;

        // for build
        _archive = null;
        _buildto = null;
        _jar = null;
        _library = new java.util.ArrayList();

        // common
        _verbose = false;
        _mode = _UNKNOWN_MODE;
    }

    //
    //  internal operations
    //


    private org.apache.tools.ant.taskdefs.Javac
    javac()
    {
        if (_javac==null) {
            // NOTE: create Javac task now as it's used for nested classpath and bootclasspath creation
            org.apache.tools.ant.Task task = super.project.createTask("javac");
            _javac = (org.apache.tools.ant.taskdefs.Javac)task;
        }

        return _javac;
    }

    private org.apache.tools.ant.taskdefs.Jar
    jar()
    {
        if (_jar==null) {
            // NOTE: create Jar task now as it's used for nested libraries creation
            org.apache.tools.ant.Task task = super.project.createTask("jar");
            _jar = (org.apache.tools.ant.taskdefs.Jar)task;
        }

        return _jar;
    }

    private void
    addLibrary()
    {
        // NOTE: add all file added in the nested "library" element to the jar
        NestedElementWithFile[] libs = (NestedElementWithFile[])_library.toArray(new NestedElementWithFile[0]);
        org.apache.tools.ant.types.FileSet set = null;
        for (int i=0;i<libs.length;i++) {
            set = new org.apache.tools.ant.types.FileSet();
            set.setFile(libs[i].getFile());
            jar().addZipGroupFileset(set);
        }
    }


    // NOTE: check required attributes
    private void
    validateAttributes()
    throws org.apache.tools.ant.BuildException
    {
        String msg = "";
        // find mode
        if ((_srcdir!=null) && (_archive==null)) {
            _mode = _COMPILE_ONLY_MODE;
            log("[validateAttributes] compile-only mode detected");
        }

        if ((_archive!=null) && (_srcdir==null)) {
            _mode = _BUILD_ONLY_MODE;
            log("[validateAttributes] build-only mode detected");
        }

        if ((_archive!=null) && (_srcdir!=null)) {
            _mode = _TWIN_MODE;
            log("[validateAttributes] twin mode detected");
        }

        if ((_srcdir==null) && (_archive==null)) {
            msg = "either srcdir or archivename attribute must be set";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_COMPILE_ONLY_MODE)!=0) && (_compileto==null)) {
            msg = "compileto attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_COMPILE_ONLY_MODE)!=0) &&
            ((!_srcdir.exists()) || (!_srcdir.isDirectory()))) {
            msg = "source directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_COMPILE_ONLY_MODE)!=0) &&
            ((!_compileto.exists()) || (!_compileto.isDirectory()))) {
            msg = "class directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_BUILD_ONLY_MODE)!=0) && (_buildto==null)) {
            msg = "buildto attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_BUILD_ONLY_MODE)!=0) &&
            ((!_buildto.exists()) || (!_buildto.isDirectory()))) {
            msg = "archive directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (((_mode&_BUILD_ONLY_MODE)==0) && (_bootclasspath==null) && (!_bcpcreated)){
            msg = "bootclasspath attribute or nested element required";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if (_packagedir!=null) {
            // build the full class dir (including package dir)
            String classdirname = _compileto.getPath()+java.io.File.separator+_packagedir;
            _classdir = new java.io.File(classdirname);
        }
    }

    private boolean
    classUpToDate()
    {
        if (_packagedir!=null) {
            // first check that class dir exists
            if ((!_classdir.exists()) || (!_classdir.isDirectory())) {
                return false;
            }

            // check that for each source file there's a newer class file
            java.io.File[] sources = FileHelper.listFiles(_srcdir);
            java.io.File[] classes = FileHelper.listFiles(_classdir);

            /*
            String srcfiles = "";
            for (int i=0;i<sources.length;i++) {
                srcfiles = srcfiles+"  "+sources[i].getName();
            }
            log("[classUpToDate] "+srcfiles);
            */

            for (int i=0;i<sources.length;i++) {
                //log("source "+i+": "+sources[i].getPath());

                // build equiv class file
                String sname = sources[i].getName();
                int idx = sname.lastIndexOf('.');
                String classname = sname.substring(0, idx)+".class";
                java.io.File classfile = new java.io.File(_classdir, classname);

                // try to find
                java.io.File foundclass = null;
                for (int j=0;j<classes.length;j++) {
                    //log("    class "+j+": "+classes[j].getPath());
                    if (classes[j].equals(classfile)) {
                        foundclass = classes[j];
                        break;
                    }
                }

                // a source is not compiled yet
                if (foundclass==null) {
                    return false;
                }

                // check timestamp
                if (sources[i].lastModified()>foundclass.lastModified()) {
                    return false;
                }
            }

            // all source files are ok
            log("[classUpToDate] source classes are uptodate: "+_srcdir.getPath());
            return true;
        }

        return false;
    }

    private boolean
    jarUpToDate(java.io.File tmpdir)
    {
        // check if some source files were compiled
        if (tmpdir.listFiles().length==0) {
            return true;
        }

        return false;
    }

    //
    //  attribute setters
    //

    final public void
    setSrcdir(java.io.File dir)
    {
        _srcdir = dir;
    }

    final public void
    setCompileto(java.io.File dir)
    {
        _compileto = dir;
    }

    final public void
    setPackagedir(String dir)
    {
        _packagedir = dir.replace('/', java.io.File.separatorChar);
    }

    final public void
    setClasspathRef(org.apache.tools.ant.types.Reference cp)
    {
        _classpath = cp;
    }

    final public void
    setClasspathref(org.apache.tools.ant.types.Reference cp)
    {
        _classpath = cp;
    }

    final public void
    setBootclasspathRef(org.apache.tools.ant.types.Reference cp)
    {
        _bootclasspath = cp;
    }

    final public void
    setBootclasspathref(org.apache.tools.ant.types.Reference cp)
    {
        _bootclasspath = cp;
    }

    final public org.apache.tools.ant.types.Path
    createClasspath()
    {
        return javac().createClasspath();
    }

    final public org.apache.tools.ant.types.Path
    createBootclasspath()
    {
        _bcpcreated = true;
        return javac().createBootclasspath();
    }

    final public void
    setArchivename(String name)
    {
        _archive = name;
    }

    final public void
    setBuildto(java.io.File dir)
    {
        _buildto = dir;
    }

    final public void
    addLibraries(org.apache.tools.ant.types.FileSet set)
    {
        jar().addZipGroupFileset(set);
    }

    final public NestedElementWithFile
    createLibrary()
    {
        NestedElementWithFile lib = new NestedElementWithFile();
        _library.add(lib);
        return lib;
    }

    final public void
    addClasses(org.apache.tools.ant.types.FileSet set)
    {
        jar().addFileset(set);
    }

    final public void
    setVerbose(boolean verbose)
    {
        _verbose = verbose;
    }

    //
    //  org.apache.tools.ant.Task
    //

    final public void
    execute()
    throws org.apache.tools.ant.BuildException
    {
        // 
        validateAttributes();

        java.io.File tmpdir = null;
        if (_mode!=_BUILD_ONLY_MODE) {
            // create temp directory
            tmpdir = FileHelper.createTempDir();
        }

        // compile sources
        // NOTE: if one class is not uptodate, then we're going to recompile every source file
        // as javac will not be able to perform its checks (due to the fact that destdir is a
        // temporary and empty directory)
        // NOTE2: this could be better handled as the classUpToDate operation could provide
        // the list of files which need to be recompiled
        // NOTE3: do not try to compile in build-only mode
        //if (_mode!=_BUILD_ONLY_MODE) {
        if ((_mode!=_BUILD_ONLY_MODE) && (!classUpToDate())) {
            // use javac task
            // by default, include subdirectories for sources
            log("[execute] compiling sources: "+_srcdir.getPath());
            org.apache.tools.ant.types.Path srcpath = new org.apache.tools.ant.types.Path(super.project);
            srcpath.createPathElement().setLocation(_srcdir);
            javac().setSrcdir(srcpath);
            javac().setDestdir(tmpdir);
            javac().setIncludes("*.java **/*.java");
            javac().setDebug(true);
            javac().setClasspathRef(_classpath);
            javac().setBootClasspathRef(_bootclasspath);
            // NOTE: add compileto to the classpath
            org.apache.tools.ant.types.Path cp = javac().createClasspath();
            cp.setPath(_compileto.getPath());

            // verbose output
            javac().setVerbose(_verbose);

            javac().execute();
        }
        
        // build jar
        // NOTE: in build-only mode, the base dir for source is not the temporary directory;
        // this is why we've separated in two cases
        if (_mode==_BUILD_ONLY_MODE) {
            // use jar task
            log("[execute] building jar (build-only): "+_buildto.getPath()+"/"+_archive);
            jar().setDestFile(new java.io.File(_buildto, _archive));
            jar().setUpdate(true);
            org.apache.tools.ant.types.FileSet set = new org.apache.tools.ant.types.FileSet();
            set.setDir(_buildto);
            set.setIncludes("*.class **/*.class");
            jar().addFileset(set);
            addLibrary();
            jar().execute();
        }
        // NOTE: do not build in compile-only
        else if ((_mode==_TWIN_MODE) && (!jarUpToDate(tmpdir))) {
            // use jar task
            log("[execute] building jar (: "+_buildto.getPath()+"/"+_archive);
            jar().setDestFile(new java.io.File(_buildto, _archive));
            jar().setUpdate(true);
            org.apache.tools.ant.types.FileSet set = new org.apache.tools.ant.types.FileSet();
            set.setDir(tmpdir);
            set.setIncludes("*.class **/*.class");
            jar().addFileset(set);
            addLibrary();
            jar().execute();
        }

        if (_mode!=_BUILD_ONLY_MODE) {
            // move files of temp dir to class dir
            FileHelper.moveFiles(tmpdir.listFiles(), _compileto);

            // delete temp directory
            FileHelper.deleteDir(tmpdir);
        }
    }
}
