// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ecm.taskdefs;

/**
 **  <p>Feeds the OpenCCM Interface Repository with an OMG IDL3 file.</p>
 **
 **  <p><bold>Parameters</bold></p>
 **  <tr>
 **    <td><bold>Attributes</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>idl3file</td>
 **    <td>The OMG IDL3 file to be compiled.</td>
 **    <td>Yes</td>
 **  </tr>
 **  <tr>
 **    <td>generateto</td>
 **    <td>Directory of the generated OMG IDL2 files.</td>
 **    <td>No, if provided then some uptodate checks are performed.</td>
 **  </tr>
 **  <tr>
 **    <td>cfgdir</td>
 **    <td>The directory corresponding to the OpenCCM_CONFIG_DIR environment variable and where the
 **        IR3-related executables of OpenCCM store and lookup  IORs, PIDs, etc. Can be set to a 
 **        temporary directory.</td>
 **    <td>Yes</td>
 **  </tr>
 **
 **  <tr>
 **    <td><bold>Properties</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>OpenCCM.install.bin.dir</td>
 **    <td>Location of the OpenCCM binaries installation directory.</td>
 **    <td>Yes</td>
 **  </tr>
 **
 **  <p>NOTE: the OpenCCM IR3 must be started before calling this task.</p>
 **
 **  @see org.objectweb.corba.build.IR3StartTask IR3StartTask
 **/
public class IR3FeedTask
extends org.apache.tools.ant.Task
{
    // params
    private java.io.File _idl3file;
    private java.io.File _generateto;
    private java.io.File _cfgdir;

    // default constructor
    public
    IR3FeedTask()
    {
        // params
        _idl3file = null;
        _generateto = null;
        _cfgdir = null;
    }

    //
    //  internal operations
    //

    // NOTE: check required attributes, properties, environment, ...
    private void
    validate()
    throws org.apache.tools.ant.BuildException
    {
        String msg = "";

        // check OS family
        if (!OSHelper.isUnix() && !OSHelper.isWindows()) {
            msg = "target os must be of unix or windows family";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check IDL3 file
        if (_idl3file==null) {
            msg = "idl3file attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check generateto
        if ((_generateto!=null) && ((!_generateto.exists()) || (!_generateto.isDirectory()))) {
            msg = "Generation directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check cfgdir
        if (_cfgdir==null) {
            msg = "cfgdir attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }
        if ((!_cfgdir.exists()) || (!_cfgdir.isDirectory())) {
            msg = "Configuration directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check "OpenCCM.install.bin.dir" property
        String bindir = getProject().getProperty("OpenCCM.install.bin.dir");
        if (bindir==null) {
            msg = "OpenCCM.install.bin.dir property must be set";
            throw new org.apache.tools.ant.BuildException(msg);
        }
    }

    private boolean
    idl3fileUpToDate(java.io.File idl3file,
                     java.io.File gendir)
    {
        // TODO: implement
        if (gendir==null) {
            return false;
        }

        // compute IDL2 file name
        String fname = idl3file.getName();
        int idx = fname.lastIndexOf('.');
        fname = fname.substring(0, idx);
        fname = fname+".idl";
        java.io.File idl2file = new java.io.File(gendir, fname);

        if (idl2file.exists() && (idl2file.lastModified()>idl3file.lastModified())) {
            return true;
        }

        return false;
    }

    private void
    ir3feed(java.io.File cfgdir,
            java.io.File idl3file)
    throws org.apache.tools.ant.BuildException
    {
        // uses exec ant task to execute the ir3_feed command
        org.apache.tools.ant.taskdefs.ExecTask cmd = null;
        cmd = (org.apache.tools.ant.taskdefs.ExecTask)getProject().createTask("exec");

        // build executable name
        String cmdname = null;
        // convert OpenCCM.install.bin.dir path
        OSHelper.pathConvert(getProject(), "os.OpenCCM.install.bin.dir", "OpenCCM.install.bin.dir");
        String bindir = getProject().getProperty("os.OpenCCM.install.bin.dir");
        if (OSHelper.isUnix()) {
            cmdname = bindir+"/ir3_feed";
        }
        else if (OSHelper.isWindows()) {
            cmdname = bindir+"\\ir3_feed.bat";
        }

        cmd.setExecutable(cmdname);

        // add environment variables
        org.apache.tools.ant.types.Environment.Variable openccm_homedir = null;
        org.apache.tools.ant.types.Environment.Variable openccm_cfgdir = null;

        // convert OpenCCM.install.dir path
        OSHelper.pathConvert(getProject(), "os.OpenCCM.install.dir", "OpenCCM.install.dir");
        String homedir = getProject().getProperty("os.OpenCCM.install.dir");
        openccm_homedir = new org.apache.tools.ant.types.Environment.Variable();
        openccm_homedir.setKey("OpenCCM_HOMEDIR");
        openccm_homedir.setValue(homedir);
        cmd.addEnv(openccm_homedir);

        openccm_cfgdir = new org.apache.tools.ant.types.Environment.Variable();
        openccm_cfgdir.setKey("OpenCCM_CONFIG_DIR");
        openccm_cfgdir.setValue(cfgdir.getPath());
        cmd.addEnv(openccm_cfgdir);

        // add IDL3 file as an argument
        org.apache.tools.ant.types.Commandline.Argument arg = cmd.createArg();
        arg.setValue(idl3file.getPath());

        // launch
        cmd.execute();
    }

    //
    //  attribute setters
    //

    final public void
    setIdl3file(java.io.File file)
    {
        _idl3file = file;
    }

    final public void
    setGenerateto(java.io.File dir)
    {
        _generateto = dir;
    }

    final public void
    setCfgdir(java.io.File cfgdir)
    {
        _cfgdir = cfgdir;
    }

    //
    //  org.apache.tools.ant.Task
    //

    final public void
    execute()
    throws org.apache.tools.ant.BuildException
    {
        validate();

        // check if OMG IDL3 is uptodate
        if (!idl3fileUpToDate(_idl3file, _generateto)) {
            log("[execute] OMG IDL3 file is not uptodate");
            ir3feed(_cfgdir, _idl3file);
        }
    }
}
