// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default and base implementation of the <tt>Components::CCMObject</tt> interface.
 **  It also implements the <tt>ExecutorSegmentBase</tt> interface (as it's the main segment)
 **  and the <tt>SessionComponent</tt> callback (as only session component are supported at
 **  present).</p>
 **/
abstract public class CCMObjectBase
extends org.omg.CORBA.LocalObject
implements org.omg.Components.CCMObjectOperations,
           org.omg.Components.ExecutorSegmentBase,
           org.omg.Components.SessionComponent,
           java.io.Serializable
{
    // executor
    static private String _class_name = "CCMObjectBase";
    transient private org.omg.Components.CCM2Context _context;
    transient private org.omg.Components.Session2Context _mono_ctx;
    transient private org.omg.Components.SegmentedSession2Context _segmented_ctx;
    transient private org.coach.ECM.ECMContext _ecm_ctx;
    private boolean _initialized;
    // NOTE: should be set by subclasses
    protected FacetSet _facets;
    protected ReceptacleSet _receptacles;
    protected SinkSet _sinks;
    protected EmitterSet _emitters;
    protected PublisherSet _publishers;

    // default constructor
    protected
    CCMObjectBase()
    {
        // executor
        _context = null;
        _mono_ctx = null;
        _segmented_ctx = null;
        _ecm_ctx = null;
        _initialized = false;
        _facets = new FacetSet();
        _receptacles = new ReceptacleSet();
        _sinks = new SinkSet();
        _emitters = new EmitterSet();
        _publishers = new PublisherSet();
    }

    //
    //  internal operations
    //

    private org.omg.CORBA.Object
    createReference(String tid, short fid, short sid)
    {
        final String opname = "createReference";
        // NOTE:
        if (_mono_ctx!=null) {
            try {
                return _mono_ctx.create_with_new_target(tid);
            } catch (org.omg.Components.IllegalState ex) {
                // should not happen
                final String msg = "FAILED (illegal state)";
                TheLogger.error(_class_name, opname, msg);
                return null;
            }
        } else if (_segmented_ctx!=null) {
            try {
                org.omg.Components.ComponentId cid = _segmented_ctx.get_component_id();
                org.omg.Components.ComponentId ncid = cid.create_with_new_target(fid, sid);
                return _segmented_ctx.create_ref_from_cid(tid, ncid);
            } catch (org.omg.Components.IllegalState ex) {
                // should not happen
                final String msg = "FAILED (illegal state)";
                TheLogger.error(_class_name, opname, msg);
                return null;
            }
        } else {
            // should not happen
            final String msg = "FAILED (no context)";
            TheLogger.error(_class_name, opname, msg);
            return null;
        }
    }

    //
    //  abstract operations
    //

    abstract protected void
    initializePorts();

    //
    //  operation for subclasses
    //

    final protected org.coach.ECM.ECMContext
    getECMContext()
    {
        return _ecm_ctx;
    }

    final protected org.omg.Components.CCM2Context
    getCCM2Context()
    {
        return _context;
    }

    final protected org.omg.Components.Session2Context
    getSession2Context()
    {
        return _mono_ctx;
    }

    final protected org.omg.Components.SegmentedSession2Context
    getSegSession2Context()
    {
        return _segmented_ctx;
    }

    //
    //  IDL:omg.org/Components/SessionComponent:1.0
    //

    final public void
    set_session_context(org.omg.Components.SessionContext ctx)
    throws org.omg.Components.CCMException
    {
        final String opname = "set_session_context";

        if (ctx instanceof org.omg.Components.CCM2Context) {
            _context = (org.omg.Components.CCM2Context)ctx;
        } else {
            // should not happen
            final String msg = "FAILED (was expecting CCM2Context: "+ctx.getClass()+")";
            TheLogger.error(_class_name, opname, msg);
        }

        if (ctx instanceof org.omg.Components.Session2Context) {
            _mono_ctx = (org.omg.Components.Session2Context)ctx;
        } else if (ctx instanceof org.omg.Components.SegmentedSession2Context) {
            _segmented_ctx = (org.omg.Components.SegmentedSession2Context)ctx;
        } else {
            // should not happen
            final String msg = "FAILED (unsupported context: "+ctx.getClass()+")";
            TheLogger.error(_class_name, opname, msg);
        }

        if (ctx instanceof org.coach.ECM.ECMContext) {
            _ecm_ctx = (org.coach.ECM.ECMContext)ctx;
        } else {
            // should not happen
            final String msg = "FAILED (was expecting ECMContext: "+ctx.getClass()+")";
            TheLogger.error(_class_name, opname, msg);
        }

        // initialize ports
        if (!_initialized) {
            initializePorts();
            // NOTE: obtain channels
            _publishers.initializeChannel(_context);
            _emitters.initializeChannel(_context);
            _initialized = true;
        } 
    }

    // NOTE: may be overriden by subclasses
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
    }

    // NOTE: may be overriden by subclasses
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
    }

    // NOTE: may be overriden by subclasses
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
    }

    //
    //  IDL:omg.org/Components/Events:1.0
    //

    final public org.omg.Components.EventConsumerBase
    get_consumer(String name)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_consumer";
        // get info
        Sink sink = _sinks.get(name);
        if (sink==null) {
            throw new org.omg.Components.InvalidName();
        }

        // create reference
        org.omg.CORBA.Object obj = createReference(sink.type_id, sink.facet_id, sink.segment_id);
        return org.omg.Components.EventConsumerBaseHelper.unchecked_narrow(obj);
    }

    final public org.omg.Components.Cookie
    subscribe(String name,
              org.omg.Components.EventConsumerBase consumer)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.ExceededConnectionLimit
    {
        final String opname = "subscribe";
        // get info
        Publisher pub = _publishers.get(name);
        if (pub==null) {
            throw new org.omg.Components.InvalidName();
        }

        // check connection type id
        if (!consumer._is_a(pub.type_id)) {
            throw new org.omg.Components.InvalidConnection();
        }

        // check connection limit
        // NOTE: how?

        // connect to channel
        org.omg.Components.Cookie ck = null;
        try {
            ck = _context.get_events().subscribe(consumer, pub.channel_id);
        } catch (org.omg.Components.InvalidChannel ex) {
            // NOTE: should not happen
            TheLogger.error(_class_name, opname, "FAILED", ex);
        }

        return ck;
    }

    final public org.omg.Components.EventConsumerBase
    unsubscribe(String name, org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection 
    {
        final String opname = "unsubscribe";
        // get info
        Publisher pub = _publishers.get(name);
        if (pub==null) {
            throw new org.omg.Components.InvalidName();
        }

        // disconnect from channel
        org.omg.Components.EventConsumerBase consumer = null;
        try {
            consumer = _context.get_events().unsubscribe(ck);
        } catch (org.omg.Components.InvalidSubscription ex) {
            throw new org.omg.Components.InvalidConnection();
        }

        return consumer;
    }

    final public void
    connect_consumer(String name,
                     org.omg.Components.EventConsumerBase consumer)
    throws org.omg.Components.InvalidName,
           org.omg.Components.AlreadyConnected, 
           org.omg.Components.InvalidConnection
    {
        final String opname = "connect_consumer";
        // get info
        Emitter emitter = _emitters.get(name);
        if (emitter==null) {
            throw new org.omg.Components.InvalidName();
        }

        // check connection type id
        if (!consumer._is_a(emitter.type_id)) {
            throw new org.omg.Components.InvalidConnection();
        }

        // check if connected
        if (emitter.cookie!=null) {
            throw new org.omg.Components.AlreadyConnected();
        }

        // obtain channel and connect
        try {
            emitter.cookie = _context.get_events().subscribe(consumer, emitter.channel_id);
        } catch (org.omg.Components.InvalidChannel ex) {
            // NOTE: should not happen
            TheLogger.error(_class_name, opname, "FAILED", ex);
        }
    }

    final public org.omg.Components.EventConsumerBase
    disconnect_consumer(String name)
    throws org.omg.Components.InvalidName,
           org.omg.Components.NoConnection
    {
        final String opname = "disconnect_consumer";
        // get info
        Emitter emitter = _emitters.get(name);
        if (emitter==null) {
            throw new org.omg.Components.InvalidName();
        }

        // check cookie validity
        if (emitter.cookie==null) {
            throw new org.omg.Components.NoConnection();
        }

        // disconnect from channel
        org.omg.Components.EventConsumerBase consumer = null;
        try {
            consumer = _context.get_events().unsubscribe(emitter.cookie);
            emitter.cookie = null;
        } catch (org.omg.Components.InvalidSubscription ex) {
            throw new org.omg.Components.NoConnection();
        }

        return consumer;
    }

    final public org.omg.Components.ConsumerDescription[]
    get_all_consumers()
    {
        final String opname = "get_all_consumers";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.ConsumerDescription[]
    get_named_consumers(String[] names)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_named_consumers";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.EmitterDescription[]
    get_all_emitters()
    {
        final String opname = "get_all_emitters";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.EmitterDescription[]
    get_named_emitters(String[] names)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_named_emitters";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.PublisherDescription[]
    get_all_publishers()
    {
        final String opname = "get_all_publishers";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.PublisherDescription[]
    get_named_publishers(String[] names)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_named_publishers";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    //  IDL:omg.org/Components/Receptacles:1.0
    //

    final public org.omg.Components.Cookie
    connect(String name, org.omg.CORBA.Object connection)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.AlreadyConnected,
           org.omg.Components.ExceededConnectionLimit
    {
        final String opname = "connect";
        // get info
        Receptacle rec = _receptacles.get(name);
        if (rec==null) {
            throw new org.omg.Components.InvalidName();
        }

        // check limit in simple case
        if (!rec.is_multiple && (rec.cookie!=null)) {
            throw new org.omg.Components.AlreadyConnected();
        }

        // check limit in multiple case
        // TODO: how?

        // check connection type id
        if (!connection._is_a(rec.type_id)) {
            throw new org.omg.Components.InvalidConnection();
        }

        // connect
        org.omg.Components.Cookie ck = _context.get_receptacles().connect(rec.name, connection);

        // store
        if (!rec.is_multiple) {
            rec.cookie = ck;
        }

        return ck;
    }

    final public org.omg.CORBA.Object
    disconnect(String name, org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.CookieRequired,
           org.omg.Components.NoConnection
    {
        final String opname = "disconnect";
        // get info
        Receptacle rec = _receptacles.get(name);
        if (rec==null) {
            throw new org.omg.Components.InvalidName();
        }

        // check if connected in simple case
        if (!rec.is_multiple && (rec.cookie==null)) {
            throw new org.omg.Components.NoConnection();
        }

        // check ck in multiple case
        if (rec.is_multiple && (ck==null)) {
            throw new org.omg.Components.CookieRequired();
        }

        // check cookie validity
        org.omg.Components.Cookie rck = null;
        if (!rec.is_multiple) {
            rck = rec.cookie;
            rec.cookie = null;
        } else {
            rck = ck;
        }

        if (rck==null) {
            throw new org.omg.Components.InvalidConnection();
        }

        // remove connection
        org.omg.CORBA.Object obj = null;
        try {
            obj = _context.get_receptacles().disconnect(rck);
        } catch (org.omg.Components.InvalidSubscription ex) {
            // NOTE: shoud not happen
            TheLogger.error(_class_name, opname, "FAILED", ex);
        }

        // check validity
        if (obj==null) {
            // NOTE: shoud not happen
            TheLogger.error(_class_name, opname, "FAILED (object is null)");
        }

        return obj;
    }

    final public org.omg.Components.ConnectionDescription[]
    get_connections(String name)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_connections";
        // get info
        Receptacle rec = _receptacles.get(name);
        if (rec==null) {
            throw new org.omg.Components.InvalidName();
        }

        // get connections
        org.omg.Components.LocalConnection[] cnxs = null;
        try {
            cnxs = _context.get_receptacles().get_connections(rec.name);
        } catch (org.omg.Components.InvalidName ex) {
            // NOTE: shoud not happen
            TheLogger.error(_class_name, opname, "FAILED", ex);
        }

        // check length
        if (cnxs.length==0) {
            return new org.omg.Components.ConnectionDescription[0];
        }

        // map
        org.omg.Components.ConnectionDescription[] descrs = null;
        if (!rec.is_multiple) {
            // NOTE: only one connection at a time for a simple receptacle
            descrs = new org.omg.Components.ConnectionDescription[1];
            descrs[0] = org.objectweb.ccm.runtime.ConnectionDescriptionFactoryImpl.create_valuetype();
            descrs[0].ck = rec.cookie;
            descrs[0].objref = cnxs[0].ref();
        } else {
            descrs = new org.omg.Components.ConnectionDescription[cnxs.length];
            for (int i=0;i<cnxs.length;i++) {
                descrs[i] = org.objectweb.ccm.runtime.ConnectionDescriptionFactoryImpl.create_valuetype();
                descrs[i].ck = cnxs[i].ck();
                descrs[i].objref = cnxs[i].ref();
            }
        }

        return descrs;
    }

    final public org.omg.Components.ReceptacleDescription[]
    get_all_receptacles()
    {
        final String opname = "get_all_receptacles";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.ReceptacleDescription[]
    get_named_receptacles(String[] names)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_named_receptacles";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    //  IDL:omg.org/Components/Navigation:1.0
    //

    final public org.omg.CORBA.Object
    provide_facet(String name)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_consumer";
        // get info
        Facet facet = _facets.get(name);
        if (facet==null) {
            throw new org.omg.Components.InvalidName();
        }

        // create reference
        return createReference(facet.type_id, facet.facet_id, facet.segment_id);
    }

    final public org.omg.Components.FacetDescription[] 
    get_all_facets()
    {
        final String opname = "get_all_facets";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.FacetDescription[]
    get_named_facets(String[] names)
    throws org.omg.Components.InvalidName
    {
        final String opname = "get_named_facets";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public boolean
    same_component(org.omg.CORBA.Object objref)
    {
        final String opname = "same_component";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    //  IDL:omg.org/Components/CCMObject:1.0
    //

    final public org.omg.CORBA.IRObject
    get_component_def()
    {
        final String opname = "get_component_def";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public org.omg.Components.CCMHome
    get_ccm_home()
    {
        return _context.get_CCM_home();
    }

    final public org.omg.Components.PrimaryKeyBase
    get_primary_key()
    throws org.omg.Components.NoKeyAvailable
    {
        throw new org.omg.Components.NoKeyAvailable();
    }

    // NOTE: may be overriden by subclasses
    public void 
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
    }

    final public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        // nothing to do as the destruction of the reference is done in the home
    }

    final public org.omg.Components.ComponentPortDescription
    get_all_ports()
    {
        final String opname = "get_all_ports";
        final String msg = "FAILED (op not impl, rethrown)";
        TheLogger.log(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }
}
